@@ -2,12 +2,13 @@
 // Use of this source code is governed by a BSD-style
 // license that can be found in the LICENSE file.
 
-//go:build !windows
-// +build !windows
+//go:build !windows && !js
+// +build !windows,!js
 
 package safesocket
 
 import (
+	"errors"
 	"fmt"
 	"io"
 	"io/ioutil"
@@ -23,6 +24,9 @@ import (
 
 // TODO(apenwarr): handle magic cookie auth
 func connect(path string, port uint16) (net.Conn, error) {
+	if runtime.GOOS == "js" {
+		return nil, errors.New("safesocket.Connect not yet implemented on js/wasm")
+	}
 	if runtime.GOOS == "darwin" && path == "" && port == 0 {
 		return connectMacOSAppSandbox()
 	}
